<?php
	function getClientIp()
	{
		if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		} elseif (!empty($_SERVER['HTTP_X_REAL_IP'])) {
			$ip = $_SERVER['HTTP_X_REAL_IP'];
		} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
			$ip = preg_replace('/,.*/', '', $ip); # hosts are comma-separated, client is first
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		return preg_replace('/^::ffff:/', '', $ip);
	}

	function getLocalOrPrivateIpInfo($ip)
	{
		if ('::1' === $ip) {
			return 'localhost IPv6 access';
		}
		if (stripos($ip, 'fe80:') === 0) {
			return 'link-local IPv6 access';
		}
		if (strpos($ip, '127.') === 0) {
			return 'localhost IPv4 access';
		}
		if (strpos($ip, '10.') === 0) {
			return 'private IPv4 access';
		}
		if (preg_match('/^172\.(1[6-9]|2\d|3[01])\./', $ip) === 1) {
			return 'private IPv4 access';
		}
		if (strpos($ip, '192.168.') === 0) {
			return 'private IPv4 access';
		}
		if (strpos($ip, '169.254.') === 0) {
			return 'link-local IPv4 access';
		}

		return null;
	}
	function sendHeaders()
	{
		header('Content-Type: application/json; charset=utf-8');
		if (isset($_GET['cors'])) {
			header('Access-Control-Allow-Origin: *');
			header('Access-Control-Allow-Methods: GET, POST');
		}
		header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0, s-maxage=0');
		header('Cache-Control: post-check=0, pre-check=0', false);
		header('Pragma: no-cache');
	}
	
	function sendResponse(
		$ip,
		$ipInfo = null,
		$distance = null,
		$rawIspInfo = null
	) {
		$processedString = $ip;
		if (is_string($ipInfo)) {
			$processedString .= ' - '.$ipInfo;
		}

		if (
			is_array($rawIspInfo)
			&& array_key_exists('country', $rawIspInfo)
		) {
			$processedString .= ', '.$rawIspInfo['country'];
		}
		if (is_string($distance)) {
			$processedString .= ' ('.$distance.')';
		}

		sendHeaders();
		echo json_encode([
			'processedString' => $processedString,
			'rawIspInfo' => $rawIspInfo ?: '',
		]);
	}

	$ip = getClientIp();

	$localIpInfo = getLocalOrPrivateIpInfo($ip);
	if (is_string($localIpInfo)) {
		sendResponse($ip, $localIpInfo);
		exit;
	}

	if (!isset($_GET['isp'])) {
		sendResponse($ip);
		exit;
	}

	sendResponse($ip, null, null, null);